@@ -1,5 +1,5 @@
 from roboticstoolbox.tools.null import null
-from roboticstoolbox.tools.p_servo import p_servo
+from roboticstoolbox.tools.p_servo import p_servo, angle_axis
 from roboticstoolbox.tools.Ticker import Ticker
 from roboticstoolbox.tools.urdf import *  # noqa
 from roboticstoolbox.tools.trajectory import (
@@ -26,6 +26,7 @@ from roboticstoolbox.tools.params import rtb_set_param, rtb_get_param
 __all__ = [
     "null",
     "p_servo",
+    "angle_axis",
     "Ticker",
     "quintic",
     "quintic_func",
